@@ -10,9 +10,11 @@ BEGIN {
 
 BEGIN {
 	$Error::TypeTiny::Assertion::AUTHORITY = 'cpan:TOBYINK';
-	$Error::TypeTiny::Assertion::VERSION   = '1.002001';
+	$Error::TypeTiny::Assertion::VERSION   = '1.010001';
 }
 
+$Error::TypeTiny::Assertion::VERSION =~ tr/_//d;
+
 require Error::TypeTiny;
 our @ISA = 'Error::TypeTiny';
 
@@ -31,7 +33,18 @@ sub new
 	my $class = shift;
 	my $self  = $class->SUPER::new(@_);
 	
-	if (ref $Method::Generate::Accessor::CurrentAttribute)
+	# Supported but undocumented parameter is `mgaca`.
+	# This indicates whether Error::TypeTiny::Assertion
+	# should attempt to figure out which attribute caused
+	# the error from Method::Generate::Accessor's info.
+	# Can be set to true/false or not set. If not set,
+	# the current behaviour is true, but this may change
+	# in the future. If set to false, will ignore the
+	# $Method::Generate::Accessor::CurrentAttribute hashref.
+	#
+	
+	if (ref $Method::Generate::Accessor::CurrentAttribute
+	and $self->{mgaca} || !exists $self->{mgaca})
 	{
 		require B;
 		my %d = %{$Method::Generate::Accessor::CurrentAttribute};
@@ -187,7 +200,7 @@ Toby Inkster E<lt>tobyink@cpan.orgE<gt>.
 
 =head1 COPYRIGHT AND LICENCE
 
-This software is copyright (c) 2013-2014, 2017 by Toby Inkster.
+This software is copyright (c) 2013-2014, 2017-2020 by Toby Inkster.
 
 This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
