@@ -6,7 +6,9 @@ use warnings;
 sub _croak ($;@) { require Error::TypeTiny; goto \&Error::TypeTiny::croak }
 
 our $AUTHORITY = 'cpan:TOBYINK';
-our $VERSION   = '1.002001';
+our $VERSION   = '1.010001';
+
+$VERSION =~ tr/_//d;
 
 # Token types
 # 
@@ -387,8 +389,8 @@ Evaluate: {
 		
 		if (@_ and $_[0] ne $r->type)
 		{
-			unshift @{$self->{stack}}, pop @{$self->{done}};
-			Type::Parser::_croak("Expected $_[0]; got ".$r->type);
+			unshift @{$self->{stack}}, pop @{$self->{done}};        # uncoverable statement
+			Type::Parser::_croak("Expected $_[0]; got ".$r->type);  # uncoverable statement
 		}
 		
 		return $r;
@@ -610,7 +612,7 @@ Toby Inkster E<lt>tobyink@cpan.orgE<gt>.
 
 =head1 COPYRIGHT AND LICENCE
 
-This software is copyright (c) 2013-2014, 2017 by Toby Inkster.
+This software is copyright (c) 2013-2014, 2017-2020 by Toby Inkster.
 
 This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
