@@ -6,9 +6,11 @@ use warnings;
 
 BEGIN {
 	$Type::Registry::AUTHORITY = 'cpan:TOBYINK';
-	$Type::Registry::VERSION   = '1.002001';
+	$Type::Registry::VERSION   = '1.010001';
 }
 
+$Type::Registry::VERSION =~ tr/_//d;
+
 use Exporter::Tiny qw( mkopt );
 use Scalar::Util qw( refaddr );
 use Type::Parser qw( eval_type );
@@ -19,21 +21,17 @@ our @EXPORT_OK = qw(t);
 
 sub _croak ($;@) { require Error::TypeTiny; goto \&Error::TypeTiny::croak }
 
-sub _exporter_expand_sub
+sub _generate_t
 {
 	my $class = shift;
-	my ($name, $value, $globals, $permitted) = @_;
+	my ($name, $value, $globals) = @_;
 	
-	if ($name eq "t")
-	{
-		my $caller = $globals->{into};
-		my $reg = $class->for_class(
-			ref($caller) ? sprintf('HASH(0x%08X)', refaddr($caller)) : $caller
-		);
-		return t => sub (;$) { @_ ? $reg->lookup(@_) : $reg };
-	}
+	my $caller = $globals->{into};
+	my $reg = $class->for_class(
+		ref($caller) ? sprintf('HASH(0x%08X)', refaddr($caller)) : $caller
+	);
 	
-	return $class->SUPER::_exporter_expand_sub(@_);
+	sub (;$) { @_ ? $reg->lookup(@_) : $reg };
 }
 
 sub new
@@ -67,7 +65,7 @@ sub add_types
 	my $opts = mkopt(\@_);
 	for my $opt (@$opts)
 	{
-		my ($library, $types) = @_;
+		my ($library, $types) = @$opt;
 		$library =~ s/^-/Types::/;
 		
 		{
@@ -170,14 +168,36 @@ sub simple_lookup
 	my ($tc) = @_;
 	$tc =~ s/(^\s+|\s+$)//g;
 	
-	if (exists $self->{$tc})
-	{
+	if (exists $self->{$tc}) {
 		return $self->{$tc};
 	}
+	elsif ($self->has_parent) {
+		return $self->get_parent->simple_lookup(@_);
+	}
 	
 	return;
 }
 
+sub set_parent {
+	my $self = shift;
+	$self->{'~~parent'} = ref($_[0]) ? $_[0] : (ref($self)||$self)->for_class($_[0]);
+	$self;
+}
+
+sub clear_parent {
+	my $self = shift;
+	delete $self->{'~~parent'};
+	$self;
+}
+
+sub has_parent {
+	!!ref(shift->{'~~parent'});
+}
+
+sub get_parent {
+	shift->{'~~parent'};
+}
+
 sub foreign_lookup
 {
 	my $self = shift;
@@ -474,6 +494,11 @@ Overloaded to call C<lookup>.
 
    $registry->Str;  # like $registry->lookup("Str")
 
+=item C<get_parent>, C<< set_parent($reg) >>, C<< clear_parent >>, C<< has_parent >>
+
+Advanced stuff. Allows a registry to have a "parent" registry which it
+inherits type constraints from.
+
 =back
 
 =head2 Functions
@@ -502,7 +527,7 @@ Toby Inkster E<lt>tobyink@cpan.orgE<gt>.
 
 =head1 COPYRIGHT AND LICENCE
 
-This software is copyright (c) 2013-2014, 2017 by Toby Inkster.
+This software is copyright (c) 2013-2014, 2017-2020 by Toby Inkster.
 
 This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
