@@ -6,9 +6,11 @@ use warnings;
 
 BEGIN {
 	$Type::Utils::AUTHORITY = 'cpan:TOBYINK';
-	$Type::Utils::VERSION   = '1.002001';
+	$Type::Utils::VERSION   = '1.010001';
 }
 
+$Type::Utils::VERSION =~ tr/_//d;
+
 sub _croak ($;@) { require Error::TypeTiny; goto \&Error::TypeTiny::croak }
 
 use Scalar::Util qw< blessed >;
@@ -59,8 +61,9 @@ sub extends
 			{
 				my $moose = Moose::Util::TypeConstraints::find_type_constraint($types->{$name});
 				my $tt    = Types::TypeTiny::to_TypeTiny($moose);
+				my $c     = $moose->has_coercion && @{ $moose->coercion->type_coercion_map || [] };
 				$caller->add_type(
-					$tt->create_child_type(library => $caller, name => $name, coercion => $moose->has_coercion ? 1 : 0)
+					$tt->create_child_type(library => $caller, name => $name, coercion => $c ? 1 : 0)
 				);
 			}
 		}
@@ -77,6 +80,18 @@ sub extends
 				);
 			}
 		}
+		elsif ($lib->isa('Specio::Exporter'))
+		{
+			my $types = $lib->Specio::Registry::exportable_types_for_package;
+			for my $name (sort keys %$types)
+			{
+				my $specio = $types->{$name};
+				my $tt     = Types::TypeTiny::to_TypeTiny($specio);
+				$caller->add_type(
+					$tt->create_child_type(library => $caller, name => $name)
+				);
+			}
+		}
 		else
 		{
 			_croak("'$lib' is not a type constraint library");
@@ -164,7 +179,7 @@ sub inline_as (&;@)
 
 sub class_type
 {
-	my $name = ref($_[0]) ? undef : shift;
+	my $name = ref($_[0]) eq 'HASH' ? undef : shift;
 	my %opts = %{ shift or {} };
 	
 	if (defined $name)
@@ -183,7 +198,7 @@ sub class_type
 
 sub role_type
 {
-	my $name = ref($_[0]) ? undef : shift;
+	my $name = ref($_[0]) eq 'HASH' ? undef : shift;
 	my %opts = %{ shift or {} };
 	
 	if (defined $name)
@@ -202,7 +217,7 @@ sub role_type
 
 sub duck_type
 {
-	my $name    = ref($_[0]) ? undef : shift;
+	my $name    = ref($_[0]) eq 'ARRAY' ? undef : shift;
 	my @methods = @{ shift or [] };
 	
 	my %opts;
@@ -217,7 +232,7 @@ sub duck_type
 
 sub enum
 {
-	my $name   = ref($_[0]) ? undef : shift;
+	my $name   = ref($_[0]) eq 'ARRAY' ? undef : shift;
 	my @values = @{ shift or [] };
 	
 	my %opts;
@@ -232,7 +247,7 @@ sub enum
 
 sub union
 {
-	my $name = ref($_[0]) ? undef : shift;
+	my $name = ref($_[0]) eq 'ARRAY' ? undef : shift;
 	my @tcs  = @{ shift or [] };
 	
 	my %opts;
@@ -247,7 +262,7 @@ sub union
 
 sub intersection
 {
-	my $name = ref($_[0]) ? undef : shift;
+	my $name = ref($_[0]) eq 'ARRAY' ? undef : shift;
 	my @tcs  = @{ shift or [] };
 	
 	my %opts;
@@ -265,6 +280,11 @@ sub declare_coercion
 	my %opts;
 	$opts{name} = shift if !ref($_[0]);
 	
+	# I don't like this; it is a hack
+	if (ref($_[0]) eq 'Type::Tiny::_DeclaredType') {
+		$opts{name} = '' . shift;
+	}
+	
 	while (HashLike->check($_[0]) and not TypeTiny->check($_[0]))
 	{
 		%opts = (%opts, %{+shift});
@@ -665,7 +685,7 @@ specify the parent type (if any) and (possibly) refine its definition.
 
    my $EvenInt = declare as Int, where { $_ % 2 == 0 };
 
-B<< NOTE: >>
+I<< NOTE: >>
 If the caller package inherits from L<Type::Library> then any non-anonymous
 types declared in the package will be automatically installed into the
 library.
@@ -747,7 +767,7 @@ significant performance improvements.
    
    warn EvenInt->inline_check('$xxx');  # demonstration
 
-B<Experimental:> your C<inline_as> block can return a list, in which case
+Your C<inline_as> block can return a list, in which case
 these will be smushed together with "&&". The first item on the list may
 be undef, in which case the undef will be replaced by the inlined parent
 type constraint. (And will throw an exception if there is no parent.)
@@ -759,10 +779,6 @@ type constraint. (And will throw an exception if there is no parent.)
          return (undef, "($_ % 2 == 0)");
       };
 
-Returning a list like this is considered experimental, is not tested very
-much, and I offer no guarantees that it will necessarily work with
-Moose/Mouse/Moo.
-
 =item C<< class_type $name, { class => $package, %options } >>
 
 =item C<< class_type { class => $package, %options } >>
@@ -1102,7 +1118,7 @@ Toby Inkster E<lt>tobyink@cpan.orgE<gt>.
 
 =head1 COPYRIGHT AND LICENCE
 
-This software is copyright (c) 2013-2014, 2017 by Toby Inkster.
+This software is copyright (c) 2013-2014, 2017-2020 by Toby Inkster.
 
 This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
